from setuptools import Command

import shlex
import subprocess
import os

WHEELHOUSE = "wheelhouse"
REQUIREMENTS_FILE = "requirements/requirements.txt"
REQUIREMENTS_FILE_BACKUP = "requirements/requirements.txt.backup"


class Package(Command):
    """Package Code and Dependencies into wheelhouse"""
    description = "Run wheels for dependencies and submodules dependencies"
    user_options = []

    def __init__(self, dist):
        Command.__init__(self, dist)

    def initialize_options(self):
        """Set default values for options."""
        pass

    def finalize_options(self):
        """Post-process options."""

    pass

    def localize_requirements(self):
        """
        After the package is unpacked at the target destination, the requirements can be installed
        locally from the wheelhouse folder using the option --no-index on pip install which
        ignores package index (only looking at --find-links URLs instead).
        --find-links <url | path> looks for archive from url or path.
        Since the original requirements.txt might have links to a non pip repo such as github
        (https) it will parse the links for the archive from a url and not from the wheelhouse.
        This functions creates a new requirements.txt with the only name and version for each of
        the packages, thus eliminating the need to fetch / parse links from http sources and install
        all archives from the wheelhouse.
        """
        dependencies = open(REQUIREMENTS_FILE).read().split("\n")
        local_dependencies = []

        for dependency in dependencies:
            if dependency:
                if "egg=" in dependency:
                    pkg_name = dependency.split("egg=")[-1]
                    local_dependencies.append(pkg_name)
                elif "git+" in dependency:
                    pkg_name = dependency.split("/")[-1].split(".")[0]
                    local_dependencies.append(pkg_name)
                else:
                    local_dependencies.append(dependency)

        print("local packages in wheel: %s", local_dependencies)
        self.execute(f"mv {REQUIREMENTS_FILE} {REQUIREMENTS_FILE_BACKUP}")

        with open(REQUIREMENTS_FILE, "w") as requirements_file:
            # filter is used to remove empty list members (None).
            requirements_file.write("\n".join(filter(None, local_dependencies)))

    def execute(self, command, capture_output=False):
        """
        The execute command will loop and keep on reading the stdout and check for the return code
        and displays the output in real time.
        """

        print(f"Running shell command: '{command}'")

        if capture_output:
            return subprocess.check_output(shlex.split(command))

        process = subprocess.Popen(shlex.split(command), stdout=subprocess.PIPE)

        while True:
            output = process.stdout.readline()

            if output == b'' and process.poll() is not None:
                break
            if output:
                print(output.strip())

        return_code = process.poll()

        if return_code != 0:
            print(f"Error running command '{command}' - exit code: {return_code}")
            raise IOError("Shell Commmand Failed")

        return return_code

    def run_commands(self, commands):
        for command in commands:
            self.execute(command)

    def restore_requirements_txt(self):
        if os.path.exists(REQUIREMENTS_FILE_BACKUP):
            print(f"Restoring original {REQUIREMENTS_FILE} file")
            commands = [
                f"rm {REQUIREMENTS_FILE}",
                f"mv {REQUIREMENTS_FILE_BACKUP} {REQUIREMENTS_FILE}"
            ]
            self.run_commands(commands)

    def run(self):
        commands = []
        commands.extend([
            "rm -rf {dir}".format(dir=WHEELHOUSE),
            "mkdir -p {dir}".format(dir=WHEELHOUSE),
            f"pip wheel --wheel-dir={WHEELHOUSE} -r {REQUIREMENTS_FILE}"
        ])

        print(f"Packing {REQUIREMENTS_FILE} into wheelhouse")
        self.run_commands(commands)
        print(f"Generating local {REQUIREMENTS_FILE}")
        self.localize_requirements()

        print(f"Packing code and wheelhouse into dist")
        self.run_command("sdist")
        self.restore_requirements_txt()
